//Update Google Event With Notifications v1.0
 
//Purpose:
//Creates or Updates a related Google event with the option to send notifications
//about the updates (Initial Invitations or Update Notifications).
 
//Action Type: On Event Save
//Prevent Default Action: Yes
 
//More info on On Event Save actions and objects here:
//https://docs.dayback.com/article/20-event-actions
 
//config variables
var calendarName = 'Google Invitations From Salesforce';
var idField = '1602358009615-1615193544';
var additionalAttendeesField = '1602539921791-8217604188';
 
//----------- You shouldn’t need to edit below this line -------------------
 
//validate calendar
var calendar = getCalendarByName(calendarName);
if(!calendar) {
  utilities.showModal('Error',calendarName + ' calendar not found. Are you logged into Google?','OK',cancelFunction);
}
else{
  //calendar validated, proceed with process
  var calendarId = calendar.id;
  //global variable for the email addresses
  var attendeeEmails = [];
  //contact Phone
  var contactPhone;
  //add emails specified in custom field
  var additionalAttendees = event[additionalAttendeesField] ? event[additionalAttendeesField].split(',') : [];
  for ( var i = 0 ; i < additionalAttendees.length ; i++){
    attendeeEmails.push(additionalAttendees[i].trim());
  }
  //start SOQL quereies, beginning with the owner
  //get the Email for the Event Owner
  getOwnerEmail(processOwnerEmail);
}
 
 
 
function getCalendarByName(name) {
  //find and validate the specified Google Calendar
  var calendars = seedcodeCalendar.get('schedules');
  for (var i = 0; i < calendars.length; i++) {
    if (calendars[i].name == calendarName) {
      return calendars[i];
    }
  }
  return false;
}
 
function getOwnerEmail(callback){
  //if resource isn't specified, current user will be the owner
  var resource = event.resource[0] ? event.resource[0] : fbk.settings.userInfo.accountFullName;
  //SOQL Select Statement Fields;
  var select = 'SELECT+Email+FROM+User';
  //SOQL Where Clause
  var where = 'WHERE+Name+=\'' + resource + '\'';
  //SOQL query
  var soql = select + '+' + where;
  performSOQL(soql, callback);
}
 
function getContactEmail(callback){
  //SOQL Select Statement Fields;
  var select = 'SELECT+Email,Phone+FROM+Contact';
  //SOQL Where Clause
  var where = 'WHERE+Name+=\'' + event.contactName[0] + '\'';
  //SOQL query
  var soql = select + '+' + where;
  performSOQL(soql, callback);
}
 
function performSOQL(soql, callback){
  //retrieve our canvas client object for authentication
  var client = fbk.client();
  //retrieve our canvas context object for REST links/urls
  var context = fbk.context();
  //retrieve the query URL from context
  var url = context.links.queryUrl;
  //final URL for GET
  var finalUrl = url + '?q=' + soql;
  //build settings object for Ajax call to Salesforce
  var settings = {};
  settings.client = client;
  settings.contentType = 'application/json';
  settings.success = callback;
  //Use canvas function to query
  Sfdc.canvas.client.ajax(finalUrl, settings);
}
 
function processOwnerEmail(data) {
  if (data && data.status == 200 && data.payload.records.length > 0) {
    //no errors querying the user, add their email to global var if there and then do the contact search
    if(data.payload.records[0].Email){
      attendeeEmails.push(data.payload.records[0].Email);
    }
    if(event.contactName.length > 0){
      //if we have a contact specified for the event, then query the contact's email
      getContactEmail(processContactEmail);
    }
    else{
      //no more emails to query, create our Google request.
      sendNotifications();
    }
  }
  else{
    //error message if owner email query error
    utilities.showModal('Salesforce error','DayBack was unable to find an email for' + event.resource[0], 'OK', cancelFunction );
  }
}
 
function processContactEmail(data) {
  if (data && data.status == 200 && data.payload.records.length > 0) {
    //no errors querying the user, add their email to global var if there and then do the contact search
    if(data.payload.records[0].Email){
      attendeeEmails.push(data.payload.records[0].Email);
      contactPhone = data.payload.records[0].Phone ? data.payload.records[0].Phone :false;
    }
    //no more emails to query, create our Google request.
    sendNotifications();
  }
  else{
    //error message if contact email query error
    utilities.showModal('Salesforce error','DayBack was unable to find an email for' + event.contactName[0], 'OK', cancelFunction );
  }
}
 
function sendNotifications(){
  //present fialog if notifications should be sent for this change
  utilities.showModal('','Would you like to send Google notifications/updates to the attendees?','Don\'t Send',dontSendFunction,'Send',sendFunction);
}
 
function sendFunction(){
  //create and send request with notifications
  createGoogleRequest(true);
}
 
function dontSendFunction(){
  //create and send request without notifications
  createGoogleRequest(false);
}
 
function createGoogleRequest(sendNotifications){
 
  //basic request
  var settings = gBk.settings;
  var request = {};
  var params = {};
  //set parameters
  params.access_token = settings.token;
  params.sendUpdates = sendNotifications ? 'all' : 'none';
 
  //map Fields
  //stuff contact info
  var contactInfo;
  if(event.contactID && event.contactID[0]){
    contactInfo = (event.contactObject ? event.contactObject : 'Contact') + ': ' + event.contactName[0];
    contactInfo = contactPhone ? contactInfo + '\n' + contactPhone : contactInfo;
  }
 
  request.description = contactInfo ? contactInfo : '';
  request.location = event.location ? event.location : '';
  request.summary = event.titleEdit ? event.titleEdit : '';
  request.end = {
    'dateTime': null,
    'date': null
  };
  request.start = {
    'dateTime': null,
    'date': null
  };
  if (event.allDay) {
    request.start.date = event.start.format('YYYY-MM-DD');
    request.end.date = event.end.format('YYYY-MM-DD');
  } else {
    request.start.dateTime = event.start.format();
    request.end.dateTime = event.end.format();
    request.start.timeZone = settings.timezone.value;
    request.end.timeZone = settings.timezone.value;
  }
  //map DayBack's extended properties as well
  request.extendedProperties = {};
  request.extendedProperties.shared = {};
  request.extendedProperties.shared.dBk_resource = JSON.stringify(event.resource);
  request.extendedProperties.shared.dBk_status = JSON.stringify(event.status);
 
  //if notifications are being sent, then need to specify those in the request.
  request.sendNotifications = true;
  request.attendees = [];
  for (var i = 0; i < attendeeEmails.length; i++) {
    request.attendees.push({
      'email': attendeeEmails[i],
    });
  }
 
  //requests to Google
  if (event[idField]) {
    //google event exists, patch it
    gBk.ajaxRequest({
      url: 'https://www.googleapis.com/calendar/' + settings.calendarApi + '/calendars/' + encodeURIComponent(calendarId) + '/events/' + encodeURIComponent(event[idField]),
      type: 'PATCH',
      params: params,
      data: request,
      retryCheck: googleRetryCheck,
      onSuccess: function(response) {
        processResponse(response);
      },
      onError: function(error) {
        googleError(error);
      },
    });
  }
  else {
    //google event doesn't exists, post it
    gBk.ajaxRequest({
      url: 'https://www.googleapis.com/calendar/' + settings.calendarApi + '/calendars/' + encodeURIComponent(calendarId) + '/events/',
      type: 'POST',
      params: params,
      data: request,
      retryCheck: googleRetryCheck,
      onSuccess: function(response) {
        processResponse(response);
      },
      onError: function(error) {
        googleError(error);
      },
    });
  }
}
 
function processResponse(data) {
  //succesful response from Google, capture the Google Id and complete Save
  changesObject[idField] = data.id;
  action.callbacks.confirm();
}
 
function googleError(error){
  //function for displaying the Google error
  utilities.showModal('Google Error', JSON.stringify(error,null,2),'OK',cancelFunction);
}
 
function googleRetryCheck(result) {
  //Retry function on Google error
  if (!result) {
    return;
  }
  var error = result.error;
  var errorResult;
 
  if (error) {
    errorResult = {
      reAuth: error && Number(error.code) === 401 || Number(error.code) === 403,
      code: error ? Number(error.code) : null,
    };
  }
  return errorResult;
}
 
function cancelFunction(){
  //dialog callback on error
  action.callbacks.cancel();
}